@@ -1,5 +1,7 @@
+{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE DeriveDataTypeable    #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE PackageImports        #-}
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE RecordWildCards       #-}
@@ -9,6 +11,7 @@
 module Network.HTTP.Download.Verified
   ( verifiedDownload
   , DownloadRequest(..)
+  , drRetryPolicyDefault
   , HashCheck(..)
   , CheckHexDigest(..)
   , LengthCheck
@@ -27,16 +30,19 @@ import Control.Monad
 import Control.Monad.Catch
 import Control.Monad.IO.Class
 import Control.Monad.Reader
+import Control.Retry (recovering,limitRetries,RetryPolicy,constantDelay)
 import Control.Applicative
-import Crypto.Hash
+import "cryptohash" Crypto.Hash
 import Crypto.Hash.Conduit (sinkHash)
 import Data.ByteString (ByteString)
+import Data.ByteString.Char8 (readInteger)
 import Data.Conduit
 import Data.Conduit.Binary (sourceHandle, sinkHandle)
 import Data.Foldable (traverse_,for_)
 import Data.Monoid
 import Data.String
 import Data.Typeable (Typeable)
+import GHC.IO.Exception (IOException(..),IOErrorType(..))
 import Network.HTTP.Client.Conduit
 import Network.HTTP.Types.Header (hContentLength, hContentMD5)
 import Path
@@ -50,8 +56,13 @@ data DownloadRequest = DownloadRequest
     { drRequest :: Request
     , drHashChecks :: [HashCheck]
     , drLengthCheck :: Maybe LengthCheck
+    , drRetryPolicy :: RetryPolicy
     }
-  deriving Show
+
+-- | Default to retrying thrice with a short constant delay.
+drRetryPolicyDefault :: RetryPolicy
+drRetryPolicyDefault = limitRetries 3 <> constantDelay onehundredMilliseconds
+  where onehundredMilliseconds = 100000
 
 data HashCheck = forall a. (Show a, HashAlgorithm a) => HashCheck
   { hashCheckAlgorithm :: a
@@ -148,7 +159,7 @@ sinkCheckHash req HashCheck{..} = do
             -- https://github.com/commercialhaskell/stack/issues/240
             || b == actualDigestHexByteString
 
-    when (not passedCheck) $
+    unless passedCheck $
         throwM $ WrongDigest req (show hashCheckAlgorithm) hashCheckHexDigest actualDigestString
 
 assertLengthSink :: MonadThrow m
@@ -178,11 +189,13 @@ hashChecksToZipSink req = traverse_ (ZipSink . sinkCheckHash req)
 -- * Verifies md5 if response includes content-md5 header
 -- * Verifies the expected hashes
 --
--- Throws VerifiedDownloadException, and whatever else "download" throws.
+-- Throws VerifiedDownloadException.
+-- Throws IOExceptions related to file system operations.
+-- Throws HttpException.
 verifiedDownload :: (MonadReader env m, HasHttpManager env, MonadIO m)
          => DownloadRequest
          -> Path Abs File -- ^ destination
-         -> Sink ByteString (ReaderT env IO) () -- ^ custom hook to observe progress
+         -> (Maybe Integer -> Sink ByteString (ReaderT env IO) ()) -- ^ custom hook to observe progress
          -> m Bool -- ^ Whether a download was performed
 verifiedDownload DownloadRequest{..} destpath progressSink = do
     let req = drRequest
@@ -190,10 +203,27 @@ verifiedDownload DownloadRequest{..} destpath progressSink = do
     liftIO $ whenM' getShouldDownload $ do
         createDirectoryIfMissing True dir
         withBinaryFile fptmp WriteMode $ \h ->
-            flip runReaderT env $
-                withResponse req (go h)
+#if MIN_VERSION_retry(0,7,0)
+            recovering drRetryPolicy handlers $ const $
+#else
+            recovering drRetryPolicy handlers $
+#endif
+                flip runReaderT env $
+                    withResponse req (go h)
         renameFile fptmp fp
   where
+    handlers = [const $ Handler alwaysRetryHttp,const $ Handler retrySomeIO]
+
+    alwaysRetryHttp :: Monad m => HttpException -> m Bool
+    alwaysRetryHttp _ = return True
+
+    retrySomeIO :: Monad m => IOException -> m Bool
+    retrySomeIO e = return $ case ioe_type e of
+                               -- hGetBuf: resource vanished (Connection reset by peer)
+                               ResourceVanished -> True
+                               -- conservatively exclude all others
+                               _ -> False
+
     whenM' mp m = do
         p <- mp
         if p then m >> return True else return False
@@ -230,7 +260,7 @@ verifiedDownload DownloadRequest{..} destpath progressSink = do
         when (fileSize /= expectedFileSize) $
             throwM $ WrongFileSize expectedFileSize fileSizeInteger
 
-    checkContentLengthHeader headers expectedContentLength = do
+    checkContentLengthHeader headers expectedContentLength =
         case List.lookup hContentLength headers of
             Just lengthBS -> do
               let lengthStr = displayByteString lengthBS
@@ -240,6 +270,10 @@ verifiedDownload DownloadRequest{..} destpath progressSink = do
 
     go h res = do
         let headers = responseHeaders res
+            mcontentLength = do
+              hLength <- List.lookup hContentLength headers
+              (i,_) <- readInteger hLength
+              return i
         for_ drLengthCheck $ checkContentLengthHeader headers
         let hashChecks = (case List.lookup hContentMD5 headers of
                 Just md5BS ->
@@ -257,4 +291,4 @@ verifiedDownload DownloadRequest{..} destpath progressSink = do
                 ( hashChecksToZipSink drRequest hashChecks
                   *> maybe (pure ()) (assertLengthSink drRequest) drLengthCheck
                   *> ZipSink (sinkHandle h)
-                  *> ZipSink progressSink)
+                  *> ZipSink (progressSink mcontentLength))
