@@ -2,6 +2,7 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE TemplateHaskell       #-}
 {-# LANGUAGE TupleSections         #-}
+{-# LANGUAGE ConstraintKinds       #-}
 -- | Cache information about previous builds
 module Stack.Build.Cache
     ( tryGetBuildCache
@@ -20,29 +21,32 @@ module Stack.Build.Cache
     , setTestSuccess
     , unsetTestSuccess
     , checkTestSuccess
+    , writePrecompiledCache
+    , readPrecompiledCache
     ) where
 
-import           Control.Exception.Enclosed (catchIO, handleIO, tryIO)
-import           Control.Monad.Catch        (MonadThrow, catch, throwM)
+import           Control.Exception.Enclosed (handleIO)
+import           Control.Monad.Catch (MonadThrow)
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger (MonadLogger)
 import           Control.Monad.Reader
-import           Data.Binary (Binary)
-import qualified Data.Binary as Binary
-import qualified Data.ByteString as S
-import qualified Data.ByteString.Lazy as L
+import qualified Crypto.Hash.SHA256 as SHA256
+import qualified Data.Binary as Binary (encode)
+import           Data.Binary.VersionTagged
+import qualified Data.ByteString.Char8 as S8
+import qualified Data.ByteString.Base16 as B16
 import           Data.Map (Map)
 import           Data.Maybe (fromMaybe, mapMaybe)
+import           Data.Set (Set)
+import qualified Data.Set as Set
+import           Data.Text (Text)
+import qualified Data.Text as T
 import           GHC.Generics (Generic)
 import           Path
 import           Path.IO
-import           Stack.Build.Types
+import           Stack.Types.Build
 import           Stack.Constants
 import           Stack.Types
-import           System.Directory           (createDirectoryIfMissing,
-                                             getDirectoryContents,
-                                             removeFile)
-import           System.IO.Error (isDoesNotExistError)
 
 -- | Directory containing files to mark an executable as installed
 exeInstalledDir :: (MonadReader env m, HasEnvConfig env, MonadThrow m)
@@ -55,15 +59,15 @@ getInstalledExes :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadThrow
                  => InstallLocation -> m [PackageIdentifier]
 getInstalledExes loc = do
     dir <- exeInstalledDir loc
-    files <- liftIO $ handleIO (const $ return []) $ getDirectoryContents $ toFilePath dir
-    return $ mapMaybe parsePackageIdentifierFromString files
+    (_, files) <- liftIO $ handleIO (const $ return ([], [])) $ listDirectory dir
+    return $ mapMaybe (parsePackageIdentifierFromString . toFilePath . filename) files
 
 -- | Mark the given executable as installed
 markExeInstalled :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadThrow m)
                  => InstallLocation -> PackageIdentifier -> m ()
 markExeInstalled loc ident = do
     dir <- exeInstalledDir loc
-    liftIO $ createDirectoryIfMissing True $ toFilePath dir
+    createTree dir
     ident' <- parseRelFile $ packageIdentifierString ident
     let fp = toFilePath $ dir </> ident'
     -- TODO consideration for the future: list all of the executables
@@ -77,8 +81,7 @@ markExeNotInstalled :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadThr
 markExeNotInstalled loc ident = do
     dir <- exeInstalledDir loc
     ident' <- parseRelFile $ packageIdentifierString ident
-    let fp = toFilePath $ dir </> ident'
-    liftIO $ catchIO (removeFile fp) (\_ -> return ())
+    removeFileIfExists (dir </> ident')
 
 -- | Stored on disk to know whether the flags have changed or any
 -- files have changed.
@@ -88,6 +91,9 @@ data BuildCache = BuildCache
     }
     deriving (Generic)
 instance Binary BuildCache
+instance HasStructuralInfo BuildCache
+instance HasSemanticVersion BuildCache
+instance NFData BuildCache
 
 -- | Try to read the dirtiness cache for the given package directory.
 tryGetBuildCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
@@ -105,21 +111,13 @@ tryGetCabalMod :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, Mo
 tryGetCabalMod = tryGetCache configCabalMod
 
 -- | Try to load a cache.
-tryGetCache :: (MonadIO m, Binary a, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+tryGetCache :: (MonadIO m, BinarySchema a)
             => (Path Abs Dir -> m (Path Abs File))
             -> Path Abs Dir
             -> m (Maybe a)
 tryGetCache get' dir = do
     fp <- get' dir
-    liftIO
-        (catch
-             (fmap (decodeMaybe . L.fromStrict) (S.readFile (toFilePath fp)))
-             (\e -> if isDoesNotExistError e
-                       then return Nothing
-                       else throwM e))
-  where decodeMaybe =
-            either (const Nothing) (Just . thd) . Binary.decodeOrFail
-          where thd (_,_,x) = x
+    decodeFileOrFailDeep fp
 
 -- | Write the dirtiness cache for this package's files.
 writeBuildCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
@@ -128,9 +126,9 @@ writeBuildCache dir times =
     writeCache
         dir
         buildCacheFile
-        (BuildCache
+        BuildCache
          { buildCacheTimes = times
-         })
+         }
 
 -- | Write the dirtiness cache for this package's configuration.
 writeConfigCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
@@ -158,17 +156,14 @@ deleteCaches dir = do
     removeFileIfExists cfp
 
 -- | Write to a cache.
-writeCache :: (Binary a, MonadIO m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env, HasEnvConfig env)
+writeCache :: (BinarySchema a, MonadIO m)
            => Path Abs Dir
            -> (Path Abs Dir -> m (Path Abs File))
            -> a
            -> m ()
 writeCache dir get' content = do
     fp <- get' dir
-    liftIO
-        (L.writeFile
-             (toFilePath fp)
-             (Binary.encode content))
+    taggedEncodeFile fp content
 
 flagCacheFile :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
               => Installed
@@ -176,7 +171,7 @@ flagCacheFile :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
 flagCacheFile installed = do
     rel <- parseRelFile $
         case installed of
-            Library gid -> ghcPkgIdString gid
+            Library _ gid -> ghcPkgIdString gid
             Executable ident -> packageIdentifierString ident
     dir <- flagCacheLocal
     return $ dir </> rel
@@ -186,11 +181,8 @@ tryGetFlagCache :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env
                 => Installed
                 -> m (Maybe ConfigCache)
 tryGetFlagCache gid = do
-    file <- flagCacheFile gid
-    eres <- liftIO $ tryIO $ Binary.decodeFileOrFail $ toFilePath file
-    case eres of
-        Right (Right x) -> return $ Just x
-        _ -> return Nothing
+    fp <- flagCacheFile gid
+    decodeFileOrFailDeep fp
 
 writeFlagCache :: (MonadIO m, MonadReader env m, HasEnvConfig env, MonadThrow m)
                => Installed
@@ -199,9 +191,8 @@ writeFlagCache :: (MonadIO m, MonadReader env m, HasEnvConfig env, MonadThrow m)
 writeFlagCache gid cache = do
     file <- flagCacheFile gid
     liftIO $ do
-        createDirectoryIfMissing True $ toFilePath $ parent file
-
-        Binary.encodeFile (toFilePath file) cache
+        createTree (parent file)
+        taggedEncodeFile file cache
 
 -- | Mark a test suite as having succeeded
 setTestSuccess :: (MonadIO m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env, HasEnvConfig env)
@@ -231,3 +222,92 @@ checkTestSuccess dir =
     liftM
         (fromMaybe False)
         (tryGetCache testSuccessFile dir)
+
+--------------------------------------
+-- Precompiled Cache
+--
+-- Idea is simple: cache information about packages built in other snapshots,
+-- and then for identical matches (same flags, config options, dependencies)
+-- just copy over the executables and reregister the libraries.
+--------------------------------------
+
+-- | The file containing information on the given package/configuration
+-- combination. The filename contains a hash of the non-directory configure
+-- options for quick lookup if there's a match.
+precompiledCacheFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
+                     => PackageIdentifier
+                     -> ConfigureOpts
+                     -> Set GhcPkgId -- ^ dependencies
+                     -> m (Path Abs File)
+precompiledCacheFile pkgident copts installedPackageIDs = do
+    ec <- asks getEnvConfig
+
+    compiler <- parseRelDir $ compilerVersionString $ envConfigCompilerVersion ec
+    cabal <- parseRelDir $ versionString $ envConfigCabalVersion ec
+    pkg <- parseRelDir $ packageIdentifierString pkgident
+
+    -- In Cabal versions 1.22 and later, the configure options contain the
+    -- installed package IDs, which is what we need for a unique hash.
+    -- Unfortunately, earlier Cabals don't have the information, so we must
+    -- supplement it with the installed package IDs directly. In 20/20
+    -- hindsight, we would simply always do that, but previous Stack releases
+    -- used only the options, and we don't want to invalidate old caches
+    -- unnecessarily.
+    --
+    -- See issue: https://github.com/commercialhaskell/stack/issues/1103
+    let cacheInput
+            | envConfigCabalVersion ec >= $(mkVersion "1.22") =
+                Binary.encode $ coNoDirs copts
+            | otherwise =
+                Binary.encode
+                    ( coNoDirs copts
+                    , installedPackageIDs
+                    )
+
+    -- We only pay attention to non-directory options. We don't want to avoid a
+    -- cache hit just because it was installed in a different directory.
+    copts' <- parseRelFile $ S8.unpack $ B16.encode $ SHA256.hashlazy cacheInput
+
+    return $ getStackRoot ec
+         </> $(mkRelDir "precompiled")
+         </> compiler
+         </> cabal
+         </> pkg
+         </> copts'
+
+-- | Write out information about a newly built package
+writePrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadIO m)
+                      => BaseConfigOpts
+                      -> PackageIdentifier
+                      -> ConfigureOpts
+                      -> Set GhcPkgId -- ^ dependencies
+                      -> Installed -- ^ library
+                      -> Set Text -- ^ executables
+                      -> m ()
+writePrecompiledCache baseConfigOpts pkgident copts depIDs mghcPkgId exes = do
+    file <- precompiledCacheFile pkgident copts depIDs
+    createTree $ parent file
+    mlibpath <-
+        case mghcPkgId of
+            Executable _ -> return Nothing
+            Library _ ipid -> liftM Just $ do
+                ipid' <- parseRelFile $ ghcPkgIdString ipid ++ ".conf"
+                return $ toFilePath $ bcoSnapDB baseConfigOpts </> ipid'
+    exes' <- forM (Set.toList exes) $ \exe -> do
+        name <- parseRelFile $ T.unpack exe
+        return $ toFilePath $ bcoSnapInstallRoot baseConfigOpts </> bindirSuffix </> name
+    liftIO $ taggedEncodeFile file PrecompiledCache
+        { pcLibrary = mlibpath
+        , pcExes = exes'
+        }
+
+-- | Check the cache for a precompiled package matching the given
+-- configuration.
+readPrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadIO m)
+                     => PackageIdentifier -- ^ target package
+                     -> ConfigureOpts
+                     -> Set GhcPkgId -- ^ dependencies
+                     -> m (Maybe PrecompiledCache)
+readPrecompiledCache pkgident copts depIDs = do
+    file <- precompiledCacheFile pkgident copts depIDs
+    decodeFileOrFailDeep file
