@@ -6,7 +6,6 @@
 {-# LANGUAGE TemplateHaskell    #-}
 {-# LANGUAGE TupleSections      #-}
 
-
 -- | Resolving a build plan for a set of packages in a given Stackage
 -- snapshot.
 
@@ -22,6 +21,7 @@ module Stack.BuildPlan
     , ToolMap
     , getToolMap
     , shadowMiniBuildPlan
+    , parseCustomMiniBuildPlan
     ) where
 
 import           Control.Applicative
@@ -34,9 +34,12 @@ import           Control.Monad.Reader (asks)
 import           Control.Monad.State.Strict      (State, execState, get, modify,
                                                   put)
 import           Control.Monad.Trans.Control (MonadBaseControl)
-import           Data.Aeson.Extended (FromJSON (..), withObject, withText, (.:))
+import qualified Crypto.Hash.SHA256 as SHA256
+import           Data.Aeson.Extended (FromJSON (..), withObject, withText, (.:), (.:?), (.!=))
 import           Data.Binary.VersionTagged (taggedDecodeOrLoad)
 import           Data.ByteString (ByteString)
+import qualified Data.ByteString.Base16 as B16
+import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 import           Data.Either (partitionEithers)
 import qualified Data.Foldable as F
@@ -46,15 +49,16 @@ import qualified Data.IntMap as IntMap
 import           Data.List (intercalate)
 import           Data.Map (Map)
 import qualified Data.Map as Map
-import           Data.Maybe (mapMaybe)
-import           Data.Monoid ((<>), Monoid (..))
+import           Data.Maybe (fromMaybe, mapMaybe)
+import           Data.Monoid
 import           Data.Set (Set)
 import qualified Data.Set as Set
 import qualified Data.Text as T
+import           Data.Text.Encoding (encodeUtf8)
 import           Data.Time (Day)
 import qualified Data.Traversable as Tr
 import           Data.Typeable (Typeable)
-import           Data.Yaml (decodeFileEither)
+import           Data.Yaml (decodeEither', decodeFileEither)
 import           Distribution.PackageDescription (GenericPackageDescription,
                                                   flagDefault, flagManual,
                                                   flagName, genPackageFlags,
@@ -67,16 +71,15 @@ import           Network.HTTP.Download
 import           Network.HTTP.Types (Status(..))
 import           Network.HTTP.Client (checkStatus)
 import           Path
+import           Path.IO
 import           Prelude -- Fix AMP warning
 import           Stack.Constants
 import           Stack.Fetch
-import           Stack.GhcPkg
 import           Stack.Package
-import           Stack.PackageIndex
 import           Stack.Types
 import           Stack.Types.StackT
-import           System.Directory (createDirectoryIfMissing)
-import           System.FilePath (takeDirectory)
+import           System.Directory (canonicalizePath)
+import qualified System.FilePath as FP
 
 data BuildPlanException
     = UnknownPackages
@@ -171,21 +174,22 @@ instance Show BuildPlanException where
 --
 -- This may fail if a target package is not present in the @BuildPlan@.
 resolveBuildPlan :: (MonadThrow m, MonadIO m, MonadReader env m, HasBuildConfig env, MonadLogger m, HasHttpManager env, MonadBaseControl IO m,MonadCatch m)
-                 => EnvOverride
-                 -> MiniBuildPlan
+                 => MiniBuildPlan
                  -> (PackageName -> Bool) -- ^ is it shadowed by a local package?
                  -> Map PackageName (Set PackageName) -- ^ required packages, and users of it
                  -> m ( Map PackageName (Version, Map FlagName Bool)
                       , Map PackageName (Set PackageName)
                       )
-resolveBuildPlan menv mbp isShadowed packages
+resolveBuildPlan mbp isShadowed packages
     | Map.null (rsUnknown rs) && Map.null (rsShadowed rs) = return (rsToInstall rs, rsUsedBy rs)
     | otherwise = do
-        cache <- getPackageCaches menv
-        let maxVer = Map.fromListWith max $ map toTuple $ Map.keys cache
+        bconfig <- asks getBuildConfig
+        let maxVer =
+                Map.fromListWith max $
+                map toTuple $
+                Map.keys (bcPackageCaches bconfig)
             unknown = flip Map.mapWithKey (rsUnknown rs) $ \ident x ->
                 (Map.lookup ident maxVer, x)
-        bconfig <- asks getBuildConfig
         throwM $ UnknownPackages
             (bcStackYaml bconfig)
             unknown
@@ -202,8 +206,11 @@ data ResolveState = ResolveState
     }
 
 toMiniBuildPlan :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, MonadThrow m, HasConfig env, MonadBaseControl IO m, MonadCatch m)
-                => BuildPlan -> m MiniBuildPlan
-toMiniBuildPlan bp = do
+                => CompilerVersion -- ^ Compiler version
+                -> Map PackageName Version -- ^ cores
+                -> Map PackageName (Version, Map FlagName Bool) -- ^ non-core packages
+                -> m MiniBuildPlan
+toMiniBuildPlan compilerVersion corePackages packages = do
     $logInfo "Caching build plan"
 
     -- Determine the dependencies of all of the packages in the build plan. We
@@ -211,13 +218,13 @@ toMiniBuildPlan bp = do
     -- package index. For those, we allow missing packages to exist, and then
     -- remove those from the list of dependencies, since there's no way we'll
     -- ever reinstall them anyway.
-    (cores, missingCores) <- addDeps True ghcVersion
-        $ fmap (, Map.empty) (siCorePackages $ bpSystemInfo bp)
+    (cores, missingCores) <- addDeps True compilerVersion
+        $ fmap (, Map.empty) corePackages
 
-    (extras, missing) <- addDeps False ghcVersion $ fmap goPP $ bpPackages bp
+    (extras, missing) <- addDeps False compilerVersion packages
 
     assert (Set.null missing) $ return MiniBuildPlan
-        { mbpGhcVersion = ghcVersion
+        { mbpCompilerVersion = compilerVersion
         , mbpPackages = Map.unions
             [ fmap (removeMissingDeps (Map.keysSet cores)) cores
             , extras
@@ -225,7 +232,6 @@ toMiniBuildPlan bp = do
             ]
         }
   where
-    ghcVersion = siGhcVersion $ bpSystemInfo bp
     goCore (PackageIdentifier name version) = (name, MiniPackageInfo
                 { mpiVersion = version
                 , mpiFlags = Map.empty
@@ -235,11 +241,6 @@ toMiniBuildPlan bp = do
                 , mpiHasLibrary = True
                 })
 
-    goPP pp =
-        ( ppVersion pp
-        , pcFlagOverrides $ ppConstraints pp
-        )
-
     removeMissingDeps cores mpi = mpi
         { mpiPackageDeps = Set.intersection cores (mpiPackageDeps mpi)
         }
@@ -247,10 +248,10 @@ toMiniBuildPlan bp = do
 -- | Add in the resolved dependencies from the package index
 addDeps :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, MonadThrow m, HasConfig env, MonadBaseControl IO m, MonadCatch m)
         => Bool -- ^ allow missing
-        -> Version -- ^ GHC version
+        -> CompilerVersion -- ^ Compiler version
         -> Map PackageName (Version, Map FlagName Bool)
         -> m (Map PackageName MiniPackageInfo, Set PackageIdentifier)
-addDeps allowMissing ghcVersion toCalc = do
+addDeps allowMissing compilerVersion toCalc = do
     menv <- getMinimalEnvOverride
     platform <- asks $ configPlatform . getConfig
     (resolvedMap, missingIdents) <-
@@ -272,12 +273,12 @@ addDeps allowMissing ghcVersion toCalc = do
                     )])
     res <- forM (Map.toList byIndex) $ \(indexName', pkgs) -> withCabalFiles indexName' pkgs
         $ \ident flags cabalBS -> do
-            gpd <- readPackageUnresolvedBS Nothing cabalBS
+            (_warnings,gpd) <- readPackageUnresolvedBS Nothing cabalBS
             let packageConfig = PackageConfig
                     { packageConfigEnableTests = False
                     , packageConfigEnableBenchmarks = False
                     , packageConfigFlags = flags
-                    , packageConfigGhcVersion = ghcVersion
+                    , packageConfigCompilerVersion = compilerVersion
                     , packageConfigPlatform = platform
                     }
                 name = packageIdentifierName ident
@@ -426,14 +427,22 @@ instance FromJSON Snapshots where
 
 -- | Load up a 'MiniBuildPlan', preferably from cache
 loadMiniBuildPlan
-    :: (MonadIO m, MonadThrow m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, MonadBaseControl IO m, MonadCatch m)
+    :: (MonadIO m, MonadThrow m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, HasGHCVariant env, MonadBaseControl IO m, MonadCatch m)
     => SnapName
     -> m MiniBuildPlan
 loadMiniBuildPlan name = do
     path <- configMiniBuildPlanCache name
-    let fp = toFilePath path
-    taggedDecodeOrLoad fp $ liftM buildPlanFixes $
-        loadBuildPlan name >>= toMiniBuildPlan
+    taggedDecodeOrLoad path $ liftM buildPlanFixes $ do
+        bp <- loadBuildPlan name
+        toMiniBuildPlan
+            (siCompilerVersion $ bpSystemInfo bp)
+            (siCorePackages $ bpSystemInfo bp)
+            (fmap goPP $ bpPackages bp)
+  where
+    goPP pp =
+        ( ppVersion pp
+        , pcFlagOverrides $ ppConstraints pp
+        )
 
 -- | Some hard-coded fixes for build plans, hopefully to be irrelevant over
 -- time.
@@ -460,14 +469,14 @@ loadBuildPlan name = do
     env <- ask
     let stackage = getStackRoot env
     file' <- parseRelFile $ T.unpack file
-    let fp = stackage </> $(mkRelDir "build-plan") </> file'
+    let fp = buildPlanDir stackage </> file'
     $logDebug $ "Decoding build plan from: " <> T.pack (toFilePath fp)
     eres <- liftIO $ decodeFileEither $ toFilePath fp
     case eres of
         Right bp -> return bp
         Left e -> do
             $logDebug $ "Decoding build plan from file failed: " <> T.pack (show e)
-            liftIO $ createDirectoryIfMissing True $ takeDirectory $ toFilePath fp
+            createTree (parent fp)
             req <- parseUrl $ T.unpack url
             $logSticky $ "Downloading " <> renderSnapName name <> " build plan ..."
             $logDebug $ "Downloading build plan from: " <> url
@@ -514,15 +523,19 @@ checkBuildPlan locals mbp gpd = do
             { packageConfigEnableTests = True
             , packageConfigEnableBenchmarks = True
             , packageConfigFlags = flags
-            , packageConfigGhcVersion = ghcVersion
+            , packageConfigCompilerVersion = compilerVersion
             , packageConfigPlatform = platform
             }
 
-    ghcVersion = mbpGhcVersion mbp
+    compilerVersion = mbpCompilerVersion mbp
 
     flagName' = fromCabalFlagName . flagName
 
-    flagOptions = map Map.fromList $ mapM getOptions $ genPackageFlags gpd
+    -- Avoid exponential complexity in flag combinations making us sad pandas.
+    -- See: https://github.com/commercialhaskell/stack/issues/543
+    maxFlagOptions = 128
+
+    flagOptions = take maxFlagOptions $ map Map.fromList $ mapM getOptions $ genPackageFlags gpd
     getOptions f
         | flagManual f = [(flagName' f, flagDefault f)]
         | flagDefault f =
@@ -571,7 +584,7 @@ instance Monoid DepError where
 
 -- | Find a snapshot and set of flags that is compatible with the given
 -- 'GenericPackageDescription'. Returns 'Nothing' if no such snapshot is found.
-findBuildPlan :: (MonadIO m, MonadCatch m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, MonadBaseControl IO m)
+findBuildPlan :: (MonadIO m, MonadCatch m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, HasGHCVariant env, MonadBaseControl IO m)
               => [GenericPackageDescription]
               -> [SnapName]
               -> m (Maybe (SnapName, Map PackageName (Map FlagName Bool)))
@@ -622,8 +635,8 @@ displayDepErrors errs =
 shadowMiniBuildPlan :: MiniBuildPlan
                     -> Set PackageName
                     -> (MiniBuildPlan, Map PackageName MiniPackageInfo)
-shadowMiniBuildPlan (MiniBuildPlan ghc pkgs0) shadowed =
-    (MiniBuildPlan ghc $ Map.fromList met, Map.fromList unmet)
+shadowMiniBuildPlan (MiniBuildPlan cv pkgs0) shadowed =
+    (MiniBuildPlan cv $ Map.fromList met, Map.fromList unmet)
   where
     pkgs1 = Map.difference pkgs0 $ Map.fromSet (\_ -> ()) shadowed
 
@@ -664,3 +677,63 @@ shadowMiniBuildPlan (MiniBuildPlan ghc pkgs0) shadowed =
                 Just True -> Left
                 Just False -> Right
                 Nothing -> assert False Right
+
+parseCustomMiniBuildPlan :: (MonadIO m, MonadCatch m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, MonadBaseControl IO m)
+                         => Path Abs File -- ^ stack.yaml file location
+                         -> T.Text -> m MiniBuildPlan
+parseCustomMiniBuildPlan stackYamlFP url0 = do
+    yamlFP <- getYamlFP url0
+
+    yamlBS <- liftIO $ S.readFile $ toFilePath yamlFP
+    let yamlHash = S8.unpack $ B16.encode $ SHA256.hash yamlBS
+    binaryFilename <- parseRelFile $ yamlHash ++ ".bin"
+    customPlanDir <- getCustomPlanDir
+    let binaryFP = customPlanDir </> $(mkRelDir "bin") </> binaryFilename
+
+    taggedDecodeOrLoad binaryFP $ do
+        cs <- either throwM return $ decodeEither' yamlBS
+        let addFlags :: PackageIdentifier -> (PackageName, (Version, Map FlagName Bool))
+            addFlags (PackageIdentifier name ver) =
+                (name, (ver, fromMaybe Map.empty $ Map.lookup name $ csFlags cs))
+        toMiniBuildPlan
+            (csCompilerVersion cs)
+            Map.empty
+            (Map.fromList $ map addFlags $ Set.toList $ csPackages cs)
+  where
+    getCustomPlanDir = do
+        root <- asks $ configStackRoot . getConfig
+        return $ root </> $(mkRelDir "custom-plan")
+
+    -- Get the path to the YAML file
+    getYamlFP url =
+        case parseUrl $ T.unpack url of
+            Just req -> getYamlFPFromReq url req
+            Nothing -> getYamlFPFromFile url
+
+    getYamlFPFromReq url req = do
+        let hashStr = S8.unpack $ B16.encode $ SHA256.hash $ encodeUtf8 url
+        hashFP <- parseRelFile $ hashStr ++ ".yaml"
+        customPlanDir <- getCustomPlanDir
+
+        let cacheFP = customPlanDir </> $(mkRelDir "yaml") </> hashFP
+        _ <- download req cacheFP
+        return cacheFP
+
+    getYamlFPFromFile url = do
+        fp <- liftIO $ canonicalizePath $ toFilePath (parent stackYamlFP) FP.</> T.unpack (fromMaybe url $
+            T.stripPrefix "file://" url <|> T.stripPrefix "file:" url)
+        parseAbsFile fp
+
+data CustomSnapshot = CustomSnapshot
+    { csCompilerVersion :: !CompilerVersion
+    , csPackages :: !(Set PackageIdentifier)
+    , csFlags :: !(Map PackageName (Map FlagName Bool))
+    }
+instance FromJSON CustomSnapshot where
+    parseJSON = withObject "CustomSnapshot" $ \o -> CustomSnapshot
+        <$> ((o .: "compiler") >>= \t ->
+                case parseCompilerVersion t of
+                    Nothing -> fail $ "Invalid compiler: " ++ T.unpack t
+                    Just compilerVersion -> return compilerVersion)
+        <*> o .: "packages"
+        <*> o .:? "flags" .!= Map.empty
