@@ -7,6 +7,7 @@
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE TemplateHaskell       #-}
 {-# LANGUAGE TupleSections         #-}
+{-# LANGUAGE PackageImports        #-}
 {-# LANGUAGE ViewPatterns          #-}
 
 -- | Functionality for downloading packages securely for cabal's usage.
@@ -22,57 +23,66 @@ module Stack.Fetch
     , withCabalLoader
     ) where
 
-import qualified Codec.Archive.Tar as Tar
-import qualified Codec.Archive.Tar.Check as Tar
-import           Codec.Compression.GZip (decompress)
+import qualified Codec.Archive.Tar              as Tar
+import qualified Codec.Archive.Tar.Check        as Tar
+import qualified Codec.Archive.Tar.Entry        as Tar
+import           Codec.Compression.GZip         (decompress)
 import           Control.Applicative
-import           Control.Concurrent.Async (Concurrently (..))
-import           Control.Concurrent.MVar.Lifted (newMVar, modifyMVar)
-import           Control.Concurrent.STM          (TVar, atomically, modifyTVar,
-                                                  newTVarIO, readTVar,
-                                                  readTVarIO, writeTVar)
-import           Control.Exception (assert)
-import           Control.Monad (liftM, when, join, unless, void)
+import           Control.Concurrent.Async       (Concurrently (..))
+import           Control.Concurrent.MVar.Lifted (modifyMVar, newMVar)
+import           Control.Concurrent.STM         (TVar, atomically, modifyTVar,
+                                                 newTVarIO, readTVar,
+                                                 readTVarIO, writeTVar)
+import           Control.Exception              (assert)
+import           Control.Monad                  (join, liftM, unless, void,
+                                                 when)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader (asks)
-import           Control.Monad.Reader (runReaderT)
+import           Control.Monad.Reader           (asks, runReaderT)
 import           Control.Monad.Trans.Control
-import           Crypto.Hash (SHA512(..))
-import           Data.ByteString (ByteString)
-import qualified Data.ByteString as S
-import qualified Data.ByteString.Lazy as L
-import           Data.Either (partitionEithers)
-import qualified Data.Foldable as F
-import           Data.Function (fix)
-import           Data.IORef (newIORef, readIORef, writeIORef)
-import           Data.List (intercalate)
-import           Data.Map (Map)
-import qualified Data.Map as Map
-import           Data.Maybe (maybeToList)
-import           Data.Monoid ((<>))
-import           Data.Set (Set)
-import qualified Data.Set as Set
-import qualified Data.Text as T
-import           Data.Text.Encoding (decodeUtf8)
-import           Data.Typeable (Typeable)
-import           Data.Word (Word64)
+import "cryptohash" Crypto.Hash                 (SHA512 (..))
+import           Data.ByteString                (ByteString)
+import qualified Data.ByteString                as S
+import qualified Data.ByteString.Lazy           as L
+import           Data.Either                    (partitionEithers)
+import qualified Data.Foldable                  as F
+import           Data.Function                  (fix)
+import           Data.IORef                     (newIORef, readIORef,
+                                                 writeIORef)
+import           Data.List                      (intercalate)
+import           Data.List.NonEmpty             (NonEmpty)
+import qualified Data.List.NonEmpty             as NE
+import           Data.Map                       (Map)
+import qualified Data.Map                       as Map
+import           Data.Maybe                     (maybeToList, catMaybes)
+import           Data.Monoid                    ((<>))
+import           Data.Set                       (Set)
+import qualified Data.Set                       as Set
+import qualified Data.Text                      as T
+import           Data.Text.Encoding             (decodeUtf8)
+import           Data.Typeable                  (Typeable)
+import           Data.Word                      (Word64)
 import           Network.HTTP.Download
 import           Path
+import           Path.IO                         (dirExists, createTree)
 import           Prelude -- Fix AMP warning
 import           Stack.GhcPkg
 import           Stack.PackageIndex
 import           Stack.Types
-import           System.Directory                (canonicalizePath,
-                                                  createDirectoryIfMissing,
-                                                  doesDirectoryExist,
-                                                  renameDirectory)
-import           System.FilePath ((<.>))
-import qualified System.FilePath as FP
-import           System.IO                       (IOMode (ReadMode),
-                                                  SeekMode (AbsoluteSeek),
-                                                  hSeek, withBinaryFile)
+import           System.Directory               (canonicalizePath,
+                                                 createDirectoryIfMissing,
+                                                 doesDirectoryExist,
+                                                 renameDirectory)
+import           System.FilePath                ((<.>))
+import qualified System.FilePath                as FP
+import           System.IO                      (IOMode (ReadMode),
+                                                 SeekMode (AbsoluteSeek), hSeek,
+                                                 withBinaryFile)
+import           System.PosixCompat             (setFileMode)
+import           Text.EditDistance              as ED
+
+type PackageCaches = Map PackageIdentifier (PackageIndex, PackageCache)
 
 data FetchException
     = Couldn'tReadIndexTarball FilePath Tar.FormatError
@@ -80,7 +90,7 @@ data FetchException
     | UnpackDirectoryAlreadyExists (Set FilePath)
     | CouldNotParsePackageSelectors [String]
     | UnknownPackageNames (Set PackageName)
-    | UnknownPackageIdentifiers (Set PackageIdentifier)
+    | UnknownPackageIdentifiers (Set PackageIdentifier) String
     deriving Typeable
 instance Exception FetchException
 
@@ -106,9 +116,10 @@ instance Show FetchException where
     show (UnknownPackageNames names) =
         "The following packages were not found in your indices: " ++
         intercalate ", " (map packageNameString $ Set.toList names)
-    show (UnknownPackageIdentifiers idents) =
+    show (UnknownPackageIdentifiers idents suggestions) =
         "The following package identifiers were not found in your indices: " ++
-        intercalate ", " (map packageIdentifierString $ Set.toList idents)
+        intercalate ", " (map packageIdentifierString $ Set.toList idents) ++
+        (if null suggestions then "" else "\n" ++ suggestions)
 
 -- | Fetch packages into the cache without unpacking
 fetchPackages :: (MonadIO m, MonadBaseControl IO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadThrow m, MonadLogger m, MonadCatch m)
@@ -170,12 +181,12 @@ unpackPackageIdents menv unpackDir mdistDir idents = do
     return $ alreadyUnpacked <> nowUnpacked
 
 data ResolvedPackage = ResolvedPackage
-    { rpCache    :: !PackageCache
-    , rpIndex    :: !PackageIndex
+    { rpCache :: !PackageCache
+    , rpIndex :: !PackageIndex
     }
 
 -- | Resolve a set of package names and identifiers into @FetchPackage@ values.
-resolvePackages :: (MonadIO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadLogger m, MonadThrow m, MonadBaseControl IO m, MonadCatch m)
+resolvePackages :: (MonadIO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
                 => EnvOverride
                 -> Set PackageIdentifier
                 -> Set PackageName
@@ -188,14 +199,11 @@ resolvePackages menv idents0 names0 = do
             go >>= either throwM return
         Right x -> return x
   where
-    go = do
-        (missingNames, missingIdents, idents) <- resolvePackagesAllowMissing menv idents0 names0
-        return $
-            case () of
-                ()
-                    | not $ Set.null missingNames -> Left $ UnknownPackageNames missingNames
-                    | not $ Set.null missingIdents -> Left $ UnknownPackageIdentifiers missingIdents
-                    | otherwise -> Right idents
+    go = r <$> resolvePackagesAllowMissing menv idents0 names0
+    r (missingNames, missingIdents, idents)
+      | not $ Set.null missingNames  = Left $ UnknownPackageNames       missingNames
+      | not $ Set.null missingIdents = Left $ UnknownPackageIdentifiers missingIdents ""
+      | otherwise                    = Right idents
 
 resolvePackagesAllowMissing
     :: (MonadIO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadLogger m, MonadThrow m, MonadBaseControl IO m, MonadCatch m)
@@ -274,39 +282,88 @@ withCabalLoader menv inner = do
     runInBase <- liftBaseWith $ \run -> return (void . run)
 
     -- TODO in the future, keep all of the necessary @Handle@s open
-    let doLookup ident = do
-            eres <- doLookup' ident
+    let doLookup :: PackageIdentifier
+                 -> IO ByteString
+        doLookup ident = do
+            cachesCurr <- liftIO $ readIORef icaches
+            eres <- lookupPackageIdentifierExact ident env cachesCurr
             case eres of
-                Right bs -> return bs
+                Just bs -> return bs
                 -- Update the cache and try again
-                Left e -> join $ modifyMVar updateRef $ \toUpdate ->
-                    if toUpdate
-                        then do
+                Nothing -> do
+                    let fuzzy = fuzzyLookupCandidates ident cachesCurr
+                        suggestions = case fuzzy of
+                            Nothing ->
+                              case typoCorrectionCandidates ident cachesCurr of
+                                  Nothing -> ""
+                                  Just cs -> "Perhaps you meant " <>
+                                    orSeparated cs <> "?"
+                            Just cs -> "Possible candidates: " <>
+                              commaSeparated (NE.map packageIdentifierString cs)
+                              <> "."
+                    join $ modifyMVar updateRef $ \toUpdate ->
+                        if toUpdate then do
                             runInBase $ do
                                 $logInfo $ T.concat
                                     [ "Didn't see "
                                     , T.pack $ packageIdentifierString ident
-                                    , " in your package indices, updating and trying again"
+                                    , " in your package indices.\n"
+                                    , "Updating and trying again."
                                     ]
                                 updateAllIndices menv
                                 caches <- getPackageCaches menv
                                 liftIO $ writeIORef icaches caches
                             return (False, doLookup ident)
-                        else return (toUpdate, throwM e)
-
-        doLookup' ident = do
-            caches <- liftIO $ readIORef icaches
-            case Map.lookup ident caches of
-                Nothing ->
-                    return $ Left $ UnknownPackageIdentifiers $ Set.singleton ident
-                Just (index, cache) -> do
-                    [bs] <- flip runReaderT env
-                          $ withCabalFiles (indexName index) [(ident, cache, ())]
-                          $ \_ _ bs -> return bs
-                    return $ Right bs
-
+                        else return (toUpdate,
+                                     throwM $ UnknownPackageIdentifiers
+                                       (Set.singleton ident) suggestions)
     inner doLookup
 
+lookupPackageIdentifierExact
+  :: HasConfig env
+  => PackageIdentifier
+  -> env
+  -> PackageCaches
+  -> IO (Maybe ByteString)
+lookupPackageIdentifierExact ident env caches =
+    case Map.lookup ident caches of
+        Nothing -> return Nothing
+        Just (index, cache) -> do
+            [bs] <- flip runReaderT env
+                  $ withCabalFiles (indexName index) [(ident, cache, ())]
+                  $ \_ _ bs -> return bs
+            return $ Just bs
+
+-- | Given package identifier and package caches, return list of packages
+-- with the same name and the same two first version number components found
+-- in the caches.
+fuzzyLookupCandidates
+  :: PackageIdentifier
+  -> PackageCaches
+  -> Maybe (NonEmpty PackageIdentifier)
+fuzzyLookupCandidates (PackageIdentifier name ver) caches =
+  let (_, zero, bigger) = Map.splitLookup zeroIdent caches
+      zeroIdent         = PackageIdentifier name $(mkVersion "0.0")
+      sameName  (PackageIdentifier n _) = n == name
+      sameMajor (PackageIdentifier _ v) = toMajorVersion v == toMajorVersion ver
+  in NE.nonEmpty . filter sameMajor $ maybe [] (pure . const zeroIdent) zero
+         <> takeWhile sameName (Map.keys bigger)
+
+-- | Try to come up with typo corrections for given package identifier using
+-- package caches. This should be called before giving up, i.e. when
+-- 'fuzzyLookupCandidates' cannot return anything.
+typoCorrectionCandidates
+  :: PackageIdentifier
+  -> PackageCaches
+  -> Maybe (NonEmpty String)
+typoCorrectionCandidates ident =
+  let getName = packageNameString . packageIdentifierName
+      name    = getName ident
+  in  NE.nonEmpty
+    . Map.keys
+    . Map.filterWithKey (const . (== 1) . damerauLevenshtein name)
+    . Map.mapKeys getName
+
 -- | Figure out where to fetch from.
 getToFetch :: (MonadThrow m, MonadIO m, MonadReader env m, HasConfig env)
            => Maybe (Path Abs Dir) -- ^ directory to unpack into, @Nothing@ means no unpack
@@ -327,7 +384,7 @@ getToFetch mdest resolvedAll = do
             case mdestDir of
                 Nothing -> return Nothing
                 Just destDir -> do
-                    exists <- liftIO $ doesDirectoryExist $ toFilePath destDir
+                    exists <- dirExists destDir
                     return $ if exists then Just destDir else Nothing
         case mexists of
             Just destDir -> return $ Right (ident, destDir)
@@ -395,9 +452,10 @@ fetchPackages' mdistDir toFetchAll = do
         let downloadReq = DownloadRequest
                 { drRequest = req
                 , drHashChecks = map toHashCheck $ maybeToList (tfSHA512 toFetch)
-                , drLengthCheck = fmap fromIntegral $ tfSize toFetch
+                , drLengthCheck = fromIntegral <$> tfSize toFetch
+                , drRetryPolicy = drRetryPolicyDefault
                 }
-        let progressSink = do
+        let progressSink _ =
                 liftIO $ runInBase $ $logInfo $ packageIdentifierText ident <> ": download"
         _ <- verifiedDownload downloadReq destpath progressSink
 
@@ -419,7 +477,19 @@ fetchPackages' mdistDir toFetchAll = do
                     wrap :: Exception e => e -> FetchException
                     wrap = Couldn'tReadPackageTarball fp . toException
                     identStr = packageIdentifierString ident
+
+                    getPerms :: Tar.Entry -> (FilePath, Tar.Permissions)
+                    getPerms e = (dest FP.</> Tar.fromTarPath (Tar.entryTarPath e),
+                                  Tar.entryPermissions e)
+
+                    filePerms :: [(FilePath, Tar.Permissions)]
+                    filePerms = catMaybes $ Tar.foldEntries (\e -> (:) (Just $ getPerms e))
+                                                            [] (const []) entries
                 Tar.unpack dest entries
+                -- Reset file permissions as they were in the tarball
+                mapM_ (\(fp', perm) -> setFileMode
+                    (FP.dropTrailingPathSeparator fp')
+                    perm) filePerms
 
                 case mdistDir of
                     Nothing -> return ()
@@ -435,9 +505,7 @@ fetchPackages' mdistDir toFetchAll = do
                             -- https://github.com/commercialhaskell/stack/issues/216
                             --
                             -- Instead, use Path which is a bit more resilient
-                            newDist' <- parseAbsDir newDist
-                            createDirectoryIfMissing True
-                                $ toFilePath $ parent newDist'
+                            createTree . parent =<< parseAbsDir newDist
                             renameDirectory oldDist newDist
 
                 let cabalFP =
@@ -472,3 +540,15 @@ parMapM_ cnt f xs0 = do
         workers 1 = Concurrently worker
         workers i = Concurrently worker *> workers (i - 1)
     liftIO $ runConcurrently $ workers cnt
+
+damerauLevenshtein :: String -> String -> Int
+damerauLevenshtein = ED.restrictedDamerauLevenshteinDistance ED.defaultEditCosts
+
+orSeparated :: NonEmpty String -> String
+orSeparated xs
+  | NE.length xs == 1 = NE.head xs
+  | NE.length xs == 2 = NE.head xs <> " or " <> NE.last xs
+  | otherwise = intercalate ", " (NE.init xs) <> ", or " <> NE.last xs
+
+commaSeparated :: NonEmpty String -> String
+commaSeparated = F.fold . NE.intersperse ", "
