@@ -1,6 +1,5 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TupleSections #-}
@@ -13,6 +12,7 @@ module Stack.PackageDump
     , DumpPackage (..)
     , conduitDumpPackage
     , ghcPkgDump
+    , ghcPkgDescribe
     , InstalledCache
     , InstalledCacheEntry (..)
     , newInstalledCache
@@ -25,16 +25,16 @@ module Stack.PackageDump
     ) where
 
 import           Control.Applicative
+import           Control.Arrow ((&&&))
 import           Control.Exception.Enclosed (tryIO)
-import           Control.Monad (when, liftM)
+import           Control.Monad (liftM)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger (MonadLogger)
 import           Control.Monad.Trans.Control
 import           Data.Attoparsec.Args
 import           Data.Attoparsec.Text as P
-import           Data.Binary (Binary)
-import           Data.Binary.VersionTagged (taggedDecodeOrLoad, taggedEncodeFile, BinarySchema (..))
+import           Data.Binary.VersionTagged
 import           Data.ByteString (ByteString)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
@@ -42,54 +42,84 @@ import           Data.Conduit
 import qualified Data.Conduit.Binary as CB
 import qualified Data.Conduit.List as CL
 import           Data.Either (partitionEithers)
-import qualified Data.Foldable as F
 import           Data.IORef
 import           Data.Map (Map)
 import qualified Data.Map as Map
-import           Data.Maybe (catMaybes)
+import           Data.Maybe (catMaybes, listToMaybe)
+import           Data.Maybe.Extra (mapMaybeM)
 import qualified Data.Set as Set
 import qualified Data.Text.Encoding as T
 import           Data.Typeable (Typeable)
 import           GHC.Generics (Generic)
 import           Path
+import           Path.IO (createTree)
+import           Path.Extra (toFilePathNoTrailingSep)
 import           Prelude -- Fix AMP warning
 import           Stack.GhcPkg
 import           Stack.Types
-import           System.Directory (createDirectoryIfMissing, getDirectoryContents, doesFileExist)
+import           System.Directory (getDirectoryContents, doesFileExist)
 import           System.Process.Read
 
 -- | Cached information on whether package have profiling libraries and haddocks.
 newtype InstalledCache = InstalledCache (IORef InstalledCacheInner)
 newtype InstalledCacheInner = InstalledCacheInner (Map GhcPkgId InstalledCacheEntry)
-    deriving Binary
-instance BinarySchema InstalledCacheInner where
-    -- Don't forget to update this if you change the datatype in any way!
-    binarySchema _ = 1
+    deriving (Binary, NFData, Generic)
+instance HasStructuralInfo InstalledCacheInner
+instance HasSemanticVersion InstalledCacheInner
 
 -- | Cached information on whether a package has profiling libraries and haddocks.
 data InstalledCacheEntry = InstalledCacheEntry
     { installedCacheProfiling :: !Bool
-    , installedCacheHaddock :: !Bool }
+    , installedCacheHaddock :: !Bool
+    , installedCacheIdent :: !PackageIdentifier }
     deriving (Eq, Generic)
 instance Binary InstalledCacheEntry
+instance HasStructuralInfo InstalledCacheEntry
+instance NFData InstalledCacheEntry
 
 -- | Call ghc-pkg dump with appropriate flags and stream to the given @Sink@, for a single database
 ghcPkgDump
     :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m, MonadThrow m)
     => EnvOverride
-    -> Maybe (Path Abs Dir) -- ^ if Nothing, use global
+    -> WhichCompiler
+    -> [Path Abs Dir] -- ^ if empty, use global
     -> Sink ByteString IO a
     -> m a
-ghcPkgDump menv mpkgDb sink = do
-    F.mapM_ (createDatabase menv) mpkgDb -- TODO maybe use some retry logic instead?
-    a <- sinkProcessStdout Nothing menv "ghc-pkg" args sink
-    return a
+ghcPkgDump = ghcPkgCmdArgs ["dump"]
+
+-- | Call ghc-pkg describe with appropriate flags and stream to the given @Sink@, for a single database
+ghcPkgDescribe
+    :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m, MonadThrow m)
+    => PackageName
+    -> EnvOverride
+    -> WhichCompiler
+    -> [Path Abs Dir] -- ^ if empty, use global
+    -> Sink ByteString IO a
+    -> m a
+ghcPkgDescribe pkgName = ghcPkgCmdArgs ["describe", "--simple-output", packageNameString pkgName]
+
+-- | Call ghc-pkg and stream to the given @Sink@, for a single database
+ghcPkgCmdArgs
+    :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m, MonadThrow m)
+    => [String]
+    -> EnvOverride
+    -> WhichCompiler
+    -> [Path Abs Dir] -- ^ if empty, use global
+    -> Sink ByteString IO a
+    -> m a
+ghcPkgCmdArgs cmd menv wc mpkgDbs sink = do
+    case reverse mpkgDbs of
+        (pkgDb:_) -> createDatabase menv wc pkgDb -- TODO maybe use some retry logic instead?
+        _ -> return ()
+    sinkProcessStdout Nothing menv (ghcPkgExeName wc) args sink
   where
     args = concat
-        [ case mpkgDb of
-            Nothing -> ["--global", "--no-user-package-db"]
-            Just pkgdb -> ["--user", "--no-user-package-db", "--package-db", toFilePath pkgdb]
-        , ["dump", "--expand-pkgroot"]
+        [ case mpkgDbs of
+              [] -> ["--global", "--no-user-package-db"]
+              _ -> ["--user", "--no-user-package-db"] ++
+                  concatMap (\pkgDb -> ["--package-db", toFilePathNoTrailingSep pkgDb]) mpkgDbs
+        , cmd
+        , ["--expand-pkgroot"]
         ]
 
 -- | Create a new, empty @InstalledCache@
@@ -98,16 +128,16 @@ newInstalledCache = liftIO $ InstalledCache <$> newIORef (InstalledCacheInner Ma
 
 -- | Load a @InstalledCache@ from disk, swallowing any errors and returning an
 -- empty cache.
-loadInstalledCache :: MonadIO m => Path Abs File -> m InstalledCache
+loadInstalledCache :: (MonadLogger m, MonadIO m) => Path Abs File -> m InstalledCache
 loadInstalledCache path = do
-    m <- taggedDecodeOrLoad (toFilePath path) (return $ InstalledCacheInner Map.empty)
-    liftIO $ fmap InstalledCache $ newIORef m
+    m <- taggedDecodeOrLoad path (return $ InstalledCacheInner Map.empty)
+    liftIO $ InstalledCache <$> newIORef m
 
 -- | Save a @InstalledCache@ to disk
 saveInstalledCache :: MonadIO m => Path Abs File -> InstalledCache -> m ()
 saveInstalledCache path (InstalledCache ref) = liftIO $ do
-    createDirectoryIfMissing True $ toFilePath $ parent path
-    readIORef ref >>= taggedEncodeFile (toFilePath path)
+    createTree (parent path)
+    readIORef ref >>= taggedEncodeFile path
 
 -- | Prune a list of possible packages down to those whose dependencies are met.
 --
@@ -124,7 +154,7 @@ pruneDeps
     -> Map name item
 pruneDeps getName getId getDepends chooseBest =
       Map.fromList
-    . (map $ \item -> (getName $ getId item, item))
+    . fmap (getName . getId &&& id)
     . loop Set.empty Set.empty []
   where
     loop foundIds usedNames foundItems dps =
@@ -159,23 +189,21 @@ sinkMatching :: Monad m
                          m
                          (Map PackageName (DumpPackage Bool Bool))
 sinkMatching reqProfiling reqHaddock allowed = do
-    dps <- CL.filter (\dp -> isAllowed (dpGhcPkgId dp) &&
+    dps <- CL.filter (\dp -> isAllowed (dpPackageIdent dp) &&
                              (not reqProfiling || dpProfiling dp) &&
                              (not reqHaddock || dpHaddock dp))
        =$= CL.consume
-    return $ pruneDeps
-        (packageIdentifierName . ghcPkgIdPackageIdentifier)
+    return $ Map.fromList $ map (packageIdentifierName . dpPackageIdent &&& id) $ Map.elems $ pruneDeps
+        id
         dpGhcPkgId
         dpDepends
         const -- Could consider a better comparison in the future
         dps
   where
-    isAllowed gid =
+    isAllowed (PackageIdentifier name version) =
         case Map.lookup name allowed of
             Just version' | version /= version' -> False
             _ -> True
-      where
-        PackageIdentifier name version = ghcPkgIdPackageIdentifier gid
 
 -- | Add profiling information to the stream of @DumpPackage@s
 addProfiling :: MonadIO m
@@ -224,11 +252,11 @@ addHaddock (InstalledCache ref) =
         let gid = dpGhcPkgId dp
         h <- case Map.lookup gid m of
             Just installed -> return (installedCacheHaddock installed)
-            Nothing | null (dpLibraries dp) -> return True
+            Nothing | not (dpHasExposedModules dp) -> return True
             Nothing -> do
                 let loop [] = return False
                     loop (ifc:ifcs) = do
-                        exists <- doesFileExist (S8.unpack ifc)
+                        exists <- doesFileExist ifc
                         if exists
                             then return True
                             else loop ifcs
@@ -238,18 +266,21 @@ addHaddock (InstalledCache ref) =
 -- | Dump information for a single package
 data DumpPackage profiling haddock = DumpPackage
     { dpGhcPkgId :: !GhcPkgId
+    , dpPackageIdent :: !PackageIdentifier
     , dpLibDirs :: ![FilePath]
     , dpLibraries :: ![ByteString]
+    , dpHasExposedModules :: !Bool
     , dpDepends :: ![GhcPkgId]
-    , dpHaddockInterfaces :: ![ByteString]
+    , dpHaddockInterfaces :: ![FilePath]
+    , dpHaddockHtml :: !(Maybe FilePath)
     , dpProfiling :: !profiling
     , dpHaddock :: !haddock
+    , dpIsExposed :: !Bool
     }
     deriving (Show, Eq, Ord)
 
 data PackageDumpException
     = MissingSingleField ByteString (Map ByteString [Line])
-    | MismatchedId PackageName Version GhcPkgId
     | Couldn'tParseField ByteString [Line]
     deriving Typeable
 instance Exception PackageDumpException
@@ -262,9 +293,6 @@ instance Show PackageDumpException where
             ]
         , map (\(k, v) -> "    " ++ show (k, v)) (Map.toList values)
         ]
-    show (MismatchedId name version gid) =
-        "Invalid id/name/version in ghc-pkg dump output: " ++
-        show (name, version, gid)
     show (Couldn'tParseField name ls) =
         "Couldn't parse the field " ++ show name ++ " from lines: " ++ show ls
 
@@ -280,10 +308,7 @@ conduitDumpPackage = (=$= CL.catMaybes) $ eachSection $ do
                 _ -> throwM $ MissingSingleField k m
         -- Can't fail: if not found, same as an empty list. See:
         -- https://github.com/fpco/stack/issues/182
-        parseM k =
-            case Map.lookup k m of
-                Just vs -> vs
-                Nothing -> []
+        parseM k = Map.findWithDefault [] k m
 
         parseDepend :: MonadThrow m => ByteString -> m (Maybe GhcPkgId)
         parseDepend "builtin_rts" = return Nothing
@@ -303,29 +328,36 @@ conduitDumpPackage = (=$= CL.catMaybes) $ eachSection $ do
             name <- parseS "name" >>= parsePackageName
             version <- parseS "version" >>= parseVersion
             ghcPkgId <- parseS "id" >>= parseGhcPkgId
-            when (PackageIdentifier name version /= ghcPkgIdPackageIdentifier ghcPkgId)
-                $ throwM $ MismatchedId name version ghcPkgId
 
             -- if a package has no modules, these won't exist
             let libDirKey = "library-dirs"
-                libDirs = parseM libDirKey
                 libraries = parseM "hs-libraries"
-                haddockInterfaces = parseM "haddock-interfaces"
-            depends <- mapM parseDepend $ parseM "depends"
+                exposedModules = parseM "exposed-modules"
+                exposed = parseM "exposed"
+            depends <- mapMaybeM parseDepend $ parseM "depends"
 
-            libDirPaths <-
-                case mapM (P.parseOnly (argsParser NoEscaping) . T.decodeUtf8) libDirs of
-                    Left{} -> throwM (Couldn'tParseField libDirKey libDirs)
-                    Right dirs -> return (concat dirs)
+            let parseQuoted key =
+                    case mapM (P.parseOnly (argsParser NoEscaping) . T.decodeUtf8) val of
+                        Left{} -> throwM (Couldn'tParseField key val)
+                        Right dirs -> return (concat dirs)
+                  where
+                    val = parseM key
+            libDirPaths <- parseQuoted libDirKey
+            haddockInterfaces <- parseQuoted "haddock-interfaces"
+            haddockHtml <- parseQuoted "haddock-html"
 
             return $ Just DumpPackage
                 { dpGhcPkgId = ghcPkgId
+                , dpPackageIdent = PackageIdentifier name version
                 , dpLibDirs = libDirPaths
                 , dpLibraries = S8.words $ S8.unwords libraries
-                , dpDepends = catMaybes (depends :: [Maybe GhcPkgId])
-                , dpHaddockInterfaces = S8.words $ S8.unwords haddockInterfaces
+                , dpHasExposedModules = not (null libraries || null exposedModules)
+                , dpDepends = depends
+                , dpHaddockInterfaces = haddockInterfaces
+                , dpHaddockHtml = listToMaybe haddockHtml
                 , dpProfiling = ()
                 , dpHaddock = ()
+                , dpIsExposed = exposed == ["True"]
                 }
 
 stripPrefixBS :: ByteString -> ByteString -> Maybe ByteString
