@@ -17,16 +17,15 @@
 -- | Dealing with the 00-index file and all its cabal files.
 module Stack.PackageIndex
     ( updateAllIndices
-    , PackageDownload (..)
-    , PackageCache (..)
     , getPackageCaches
     ) where
 
 import qualified Codec.Archive.Tar as Tar
 import           Control.Exception (Exception)
 import           Control.Exception.Enclosed (tryIO)
-import           Control.Monad (unless, when, liftM, mzero)
+import           Control.Monad (unless, when, liftM)
 import           Control.Monad.Catch (MonadThrow, throwM, MonadCatch)
+import qualified Control.Monad.Catch as C
 import           Control.Monad.IO.Class (MonadIO, liftIO)
 import           Control.Monad.Logger                  (MonadLogger, logDebug,
                                                         logInfo, logWarn)
@@ -34,9 +33,7 @@ import           Control.Monad.Reader (asks)
 import           Control.Monad.Trans.Control
 
 import           Data.Aeson.Extended
-import qualified Data.Binary as Binary
-import           Data.Binary.VersionTagged (taggedDecodeOrLoad, BinarySchema (..))
-import           Data.ByteString (ByteString)
+import           Data.Binary.VersionTagged
 import qualified Data.Word8 as Word8
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Unsafe as SU
@@ -54,62 +51,45 @@ import           Data.Monoid
 import           Data.Text (Text)
 import qualified Data.Text as T
 
-
-import           Data.Text.Encoding (encodeUtf8)
-
 import           Data.Traversable (forM)
 
 import           Data.Typeable (Typeable)
 
-import           Data.Word (Word64)
-
-
-
-import           GHC.Generics (Generic)
 
 import           Network.HTTP.Download
 import           Path                                  (mkRelDir, parent,
                                                         parseRelDir, toFilePath,
-                                                        (</>))
+                                                        parseAbsFile, (</>))
 import           Path.IO
 import           Prelude -- Fix AMP warning
 import           Stack.Types
 import           Stack.Types.StackT
-import           System.Directory
 import           System.FilePath (takeBaseName, (<.>))
 import           System.IO                             (IOMode (ReadMode, WriteMode),
                                                         withBinaryFile)
 import           System.Process.Read (readInNull, EnvOverride, doesExecutableExist)
 
-data PackageCache = PackageCache
-    { pcOffset :: !Int64
-    -- ^ offset in bytes into the 00-index.tar file for the .cabal file contents
-    , pcSize :: !Int64
-    -- ^ size in bytes of the .cabal file
-    , pcDownload :: !(Maybe PackageDownload)
-    }
-    deriving Generic
-instance Binary.Binary PackageCache
-
-newtype PackageCacheMap = PackageCacheMap (Map PackageIdentifier PackageCache)
-    deriving Binary.Binary
-instance BinarySchema PackageCacheMap where
-    -- Don't forget to update this if you change the datatype in any way!
-    binarySchema _ = 1
 -- | Populate the package index caches and return them.
 populateCache
-    :: (MonadIO m, MonadThrow m, MonadReader env m, HasConfig env, HasHttpManager env, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+    :: (MonadIO m, MonadReader env m, HasConfig env, HasHttpManager env, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
     => EnvOverride
     -> PackageIndex
     -> m (Map PackageIdentifier PackageCache)
 populateCache menv index = do
-    $logSticky "Populating index cache ..."
     requireIndex menv index
     -- This uses full on lazy I/O instead of ResourceT to provide some
     -- protections. Caveat emptor
     path <- configPackageIndex (indexName index)
-    lbs <- liftIO $ L.readFile $ Path.toFilePath path
-    pis <- loop 0 Map.empty (Tar.read lbs)
+    let loadPIS = do
+            $logSticky "Populating index cache ..."
+            lbs <- liftIO $ L.readFile $ Path.toFilePath path
+            loop 0 Map.empty (Tar.read lbs)
+    pis <- loadPIS `C.catch` \e -> do
+        $logWarn $ "Exception encountered when parsing index tarball: "
+                <> T.pack (show (e :: Tar.FormatError))
+        $logWarn "Automatically updating index and trying again"
+        updateIndex menv index
+        loadPIS
 
     when (indexRequireHashes index) $ forM_ (Map.toList pis) $ \(ident, pc) ->
         case pcDownload pc of
@@ -234,7 +214,7 @@ updateIndex menv index =
         (True, ILGitHttp url _) -> logUpdate url >> updateIndexGit menv name index url
         (_, ILHttp url) -> logUpdate url >> updateIndexHTTP name index url
         (False, ILGitHttp _ url) -> logUpdate url >> updateIndexHTTP name index url
-        (False, ILGit url) -> logUpdate url >> (throwM $ GitNotAvailable name)
+        (False, ILGit url) -> logUpdate url >> throwM (GitNotAvailable name)
 
 -- | Update the index Git repo and the index tarball
 updateIndexGit :: (MonadIO m,MonadLogger m,MonadThrow m,MonadReader env m,HasConfig env,MonadBaseControl IO m, MonadCatch m)
@@ -246,7 +226,7 @@ updateIndexGit :: (MonadIO m,MonadLogger m,MonadThrow m,MonadReader env m,HasCon
 updateIndexGit menv indexName' index gitUrl = do
      tarFile <- configPackageIndex indexName'
      let idxPath = parent tarFile
-     liftIO (createDirectoryIfMissing True (toFilePath idxPath))
+     createTree idxPath
      do
             repoName <- parseRelDir $ takeBaseName $ T.unpack gitUrl
             let cloneArgs =
@@ -262,25 +242,23 @@ updateIndexGit menv indexName' index gitUrl = do
                   sDir </>
                   $(mkRelDir "git-update")
                 acfDir = suDir </> repoName
-            repoExists <-
-              liftIO (doesDirectoryExist (toFilePath acfDir))
+            repoExists <- dirExists acfDir
             unless repoExists
                    (readInNull suDir "git" menv cloneArgs Nothing)
             $logSticky "Fetching package index ..."
             readInNull acfDir "git" menv ["fetch","--tags","--depth=1"] Nothing
             $logStickyDone "Fetched package index."
-            _ <-
-              (liftIO . tryIO) (removeFile (toFilePath tarFile))
+            removeFileIfExists tarFile
             when (indexGpgVerify index)
-                 (do readInNull acfDir
-                                "git"
-                                menv
-                                ["tag","-v","current-hackage"]
-                                (Just (T.unlines ["Signature verification failed. "
-                                                 ,"Please ensure you've set up your"
-                                                 ,"GPG keychain to accept the D6CF60FD signing key."
-                                                 ,"For more information, see:"
-                                                 ,"https://github.com/fpco/stackage-update#readme"])))
+                 (readInNull acfDir
+                             "git"
+                             menv
+                             ["tag","-v","current-hackage"]
+                             (Just (T.unlines ["Signature verification failed. "
+                                              ,"Please ensure you've set up your"
+                                              ,"GPG keychain to accept the D6CF60FD signing key."
+                                              ,"For more information, see:"
+                                              ,"https://github.com/fpco/stackage-update#readme"])))
             $logDebug ("Exporting a tarball to " <>
                        (T.pack . toFilePath) tarFile)
             deleteCache indexName'
@@ -294,7 +272,8 @@ updateIndexGit menv indexName' index gitUrl = do
                        ,tarFileTmp
                        ,"current-hackage"]
                        Nothing
-            liftIO $ renameFile tarFileTmp (toFilePath tarFile)
+            tarFileTmpPath <- parseAbsFile tarFileTmp
+            renameFile tarFileTmpPath tarFile
 
 -- | Update the index tarball via HTTP
 updateIndexHTTP :: (MonadIO m,MonadLogger m
@@ -316,6 +295,7 @@ updateIndexHTTP indexName' index url = do
 
     when toUnpack $ do
         let tmp = toFilePath tar <.> "tmp"
+        tmpPath <- parseAbsFile tmp
 
         deleteCache indexName'
 
@@ -325,7 +305,7 @@ updateIndexHTTP indexName' index url = do
                     sourceHandle input
                     $$ ungzip
                     =$ sinkHandle output
-            renameFile tmp $ toFilePath tar
+            renameFile tmpPath tar
 
     when (indexGpgVerify index)
         $ $logWarn
@@ -341,34 +321,11 @@ isGitInstalled = flip doesExecutableExist "git"
 -- | Delete the package index cache
 deleteCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadLogger m, MonadThrow m) => IndexName -> m ()
 deleteCache indexName' = do
-    fp <- liftM toFilePath $ configPackageIndexCache indexName'
+    fp <- configPackageIndexCache indexName'
     eres <- liftIO $ tryIO $ removeFile fp
     case eres of
         Left e -> $logDebug $ "Could not delete cache: " <> T.pack (show e)
-        Right () -> $logDebug $ "Deleted index cache at " <> T.pack fp
-
-data PackageDownload = PackageDownload
-    { pdSHA512 :: !ByteString
-    , pdUrl    :: !ByteString
-    , pdSize   :: !Word64
-    }
-    deriving (Show, Generic)
-instance Binary.Binary PackageDownload
-instance FromJSON PackageDownload where
-    parseJSON = withObject "Package" $ \o -> do
-        hashes <- o .: "package-hashes"
-        sha512 <- maybe mzero return (Map.lookup ("SHA512" :: Text) hashes)
-        locs <- o .: "package-locations"
-        url <-
-            case reverse locs of
-                [] -> mzero
-                x:_ -> return x
-        size <- o .: "package-size"
-        return PackageDownload
-            { pdSHA512 = encodeUtf8 sha512
-            , pdUrl = encodeUtf8 url
-            , pdSize = size
-            }
+        Right () -> $logDebug $ "Deleted index cache at " <> T.pack (toFilePath fp)
 
 -- | Load the cached package URLs, or created the cache if necessary.
 getPackageCaches :: (MonadIO m, MonadLogger m, MonadReader env m, HasConfig env, MonadThrow m, HasHttpManager env, MonadBaseControl IO m, MonadCatch m)
@@ -377,7 +334,7 @@ getPackageCaches :: (MonadIO m, MonadLogger m, MonadReader env m, HasConfig env,
 getPackageCaches menv = do
     config <- askConfig
     liftM mconcat $ forM (configPackageIndices config) $ \index -> do
-        fp <- liftM toFilePath $ configPackageIndexCache (indexName index)
+        fp <- configPackageIndexCache (indexName index)
         PackageCacheMap pis' <- taggedDecodeOrLoad fp $ liftM PackageCacheMap $ populateCache menv index
 
         return (fmap (index,) pis')
