@@ -8,7 +8,14 @@ import Stack.Types.BuildPlan as X
 import Stack.Types.FlagName as X
 import Stack.Types.GhcPkgId as X
 import Stack.Types.PackageIdentifier as X
+import Stack.Types.PackageIndex as X
 import Stack.Types.PackageName as X
 import Stack.Types.Version as X
 import Stack.Types.Config as X
 import Stack.Types.Docker as X
+import Stack.Types.Nix as X
+import Stack.Types.Image as X
+import Stack.Types.Build as X
+import Stack.Types.Package as X
+import Stack.Types.Compiler as X
+import Stack.Types.Sig as X
