@@ -22,7 +22,6 @@ module Stack.Types.BuildPlan
     , MiniPackageInfo (..)
     , renderSnapName
     , parseSnapName
-    , isWindows
     ) where
 
 import           Control.Applicative
@@ -32,8 +31,7 @@ import           Control.Monad.Catch             (MonadThrow, throwM)
 import           Data.Aeson                      (FromJSON (..), ToJSON (..),
                                                   object, withObject, withText,
                                                   (.!=), (.:), (.:?), (.=))
-import           Data.Binary                     as Binary (Binary)
-import           Data.Binary.VersionTagged       (BinarySchema (..))
+import           Data.Binary.VersionTagged
 import           Data.ByteString                 (ByteString)
 import qualified Data.ByteString.Char8           as S8
 import           Data.Hashable                   (Hashable)
@@ -58,6 +56,7 @@ import qualified Distribution.Version            as C
 import           GHC.Generics                    (Generic)
 import           Prelude -- Fix AMP warning
 import           Safe (readMay)
+import           Stack.Types.Compiler
 import           Stack.Types.FlagName
 import           Stack.Types.PackageName
 import           Stack.Types.Version
@@ -211,7 +210,7 @@ instance FromJSON TestState where
                $ map (\x -> (testStateToText x, x)) [minBound..maxBound]
 
 data SystemInfo = SystemInfo
-    { siGhcVersion      :: Version
+    { siCompilerVersion :: CompilerVersion
     , siOS              :: OS
     , siArch            :: Arch
     , siCorePackages    :: Map PackageName Version
@@ -219,9 +218,11 @@ data SystemInfo = SystemInfo
     }
     deriving (Show, Eq, Ord)
 instance ToJSON SystemInfo where
-    toJSON SystemInfo {..} = object
-        [ "ghc-version" .= siGhcVersion
-        , "os" .= display siOS
+    toJSON SystemInfo {..} = object $
+        (case siCompilerVersion of
+            GhcVersion version -> "ghc-version" .= version
+            _ -> "compiler-version" .= siCompilerVersion) :
+        [ "os" .= display siOS
         , "arch" .= display siArch
         , "core-packages" .= siCorePackages
         , "core-executables" .= siCoreExecutables
@@ -229,7 +230,14 @@ instance ToJSON SystemInfo where
 instance FromJSON SystemInfo where
     parseJSON = withObject "SystemInfo" $ \o -> do
         let helper name = (o .: name) >>= either (fail . show) return . simpleParse
-        siGhcVersion <- o .: "ghc-version"
+        ghcVersion <- o .:? "ghc-version"
+        compilerVersion <- o .:? "compiler-version"
+        siCompilerVersion <-
+            case (ghcVersion, compilerVersion) of
+                (Just _, Just _) -> fail "can't have both compiler-version and ghc-version fields"
+                (Just ghc, _) -> return (GhcVersion ghc)
+                (_, Just compiler) -> return compiler
+                _ -> fail "expected field \"ghc-version\" or \"compiler-version\" not present"
         siOS <- helper "os"
         siArch <- helper "arch"
         siCorePackages <- o .: "core-packages"
@@ -241,8 +249,8 @@ newtype Maintainer = Maintainer { unMaintainer :: Text }
 
 -- | Name of an executable.
 newtype ExeName = ExeName { unExeName :: ByteString }
-    deriving (Show, Eq, Ord, Hashable, IsString, Generic)
-instance Binary ExeName
+    deriving (Show, Eq, Ord, Hashable, IsString, Generic, Binary, NFData)
+instance HasStructuralInfo ExeName
 instance ToJSON ExeName where
     toJSON = toJSON . S8.unpack . unExeName
 instance FromJSON ExeName where
@@ -361,14 +369,14 @@ instance FromJSON a => FromJSON (Map ExeName a) where
 
 -- | A simplified version of the 'BuildPlan' + cabal file.
 data MiniBuildPlan = MiniBuildPlan
-    { mbpGhcVersion :: !Version
+    { mbpCompilerVersion :: !CompilerVersion
     , mbpPackages :: !(Map PackageName MiniPackageInfo)
     }
     deriving (Generic, Show, Eq)
-instance Binary.Binary MiniBuildPlan
-instance BinarySchema MiniBuildPlan where
-    -- Don't forget to update this if you change the datatype in any way!
-    binarySchema _ = 1
+instance Binary MiniBuildPlan
+instance NFData MiniBuildPlan
+instance HasStructuralInfo MiniBuildPlan
+instance HasSemanticVersion MiniBuildPlan
 
 -- | Information on a single package for the 'MiniBuildPlan'.
 data MiniPackageInfo = MiniPackageInfo
@@ -386,10 +394,6 @@ data MiniPackageInfo = MiniPackageInfo
     -- ^ Is there a library present?
     }
     deriving (Generic, Show, Eq)
-instance Binary.Binary MiniPackageInfo
-
-
-isWindows :: OS -> Bool
-isWindows Windows = True
-isWindows (OtherOS "windowsintegersimple") = True
-isWindows _ = False
+instance Binary MiniPackageInfo
+instance HasStructuralInfo MiniPackageInfo
+instance NFData MiniPackageInfo
