@@ -10,10 +10,12 @@ import Data.Maybe
 import Data.Monoid
 import Network.HTTP.Conduit (Manager)
 import Path
+import Path.IO
 --import System.FilePath
+import Prelude -- Fix redundant import warnings
 import System.Directory
-import System.IO.Temp
 import System.Environment
+import System.IO.Temp (withSystemTempDirectory)
 import Test.Hspec
 
 import Stack.Config
@@ -60,8 +62,8 @@ spec = beforeAll setup $ afterAll teardown $ do
 
 
   describe "loadConfig" $ do
-    let loadConfig' m = runStackLoggingT m logLevel False (loadConfig mempty Nothing)
-    let loadBuildConfigRest m = runStackLoggingT m logLevel False
+    let loadConfig' m = runStackLoggingT m logLevel False False (loadConfig mempty Nothing Nothing)
+    let loadBuildConfigRest m = runStackLoggingT m logLevel False False
     -- TODO(danburton): make sure parent dirs also don't have config file
     it "works even if no config file exists" $ \T{..} -> example $ do
       _config <- loadConfig' manager
@@ -79,20 +81,20 @@ spec = beforeAll setup $ afterAll teardown $ do
       createDirectory childDir
       setCurrentDirectory childDir
       LoadConfig{..} <- loadConfig' manager
-      BuildConfig{..} <- loadBuildConfigRest manager
-                            (lcLoadBuildConfig Nothing ThrowException)
-      bcRoot `shouldBe` parentDir
+      bc@BuildConfig{..} <- loadBuildConfigRest manager
+                            (lcLoadBuildConfig Nothing)
+      bcRoot bc `shouldBe` parentDir
 
     it "respects the STACK_YAML env variable" $ \T{..} -> inTempDir $ do
-      withSystemTempDirectory "config-is-here" $ \dirFilePath -> do
-        dir <- parseAbsDir dirFilePath
+      withCanonicalizedSystemTempDirectory "config-is-here" $ \dir -> do
         let stackYamlFp = toFilePath (dir </> stackDotYaml)
         writeFile stackYamlFp sampleConfig
         withEnvVar "STACK_YAML" stackYamlFp $ do
           LoadConfig{..} <- loadConfig' manager
           BuildConfig{..} <- loadBuildConfigRest manager
-                                (lcLoadBuildConfig Nothing ThrowException)
-          bcRoot `shouldBe` dir
+                                (lcLoadBuildConfig Nothing)
+          bcStackYaml `shouldBe` dir </> stackDotYaml
+          parent bcStackYaml `shouldBe` dir
 
     it "STACK_YAML can be relative" $ \T{..} -> inTempDir $ do
         parentDir <- getCurrentDirectory >>= parseAbsDir
@@ -104,5 +106,5 @@ spec = beforeAll setup $ afterAll teardown $ do
         withEnvVar "STACK_YAML" (toFilePath yamlRel) $ do
             LoadConfig{..} <- loadConfig' manager
             BuildConfig{..} <- loadBuildConfigRest manager
-                                (lcLoadBuildConfig Nothing ThrowException)
+                                (lcLoadBuildConfig Nothing)
             bcStackYaml `shouldBe` yamlAbs
