@@ -1,31 +1,33 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TupleSections #-}
 -- | Test suite for Stack.Dot
 module Stack.DotSpec where
 
+import           Control.Monad (filterM)
 import           Data.ByteString.Char8 (ByteString)
-import qualified Data.Foldable as F
+import           Data.Foldable as F
 import           Data.Functor.Identity
 import           Data.List ((\\))
 import qualified Data.Map as Map
 import           Data.Maybe (fromMaybe)
 import           Data.Set (Set)
 import qualified Data.Set as Set
-import           Options.Applicative (execParserPure,idm,prefs,info,getParseResult)
 import           Stack.Types
 import           Test.Hspec
 import           Test.Hspec.QuickCheck (prop)
-#if MIN_VERSION_QuickCheck(2,8,0)
-import           Test.QuickCheck (forAll,sublistOf)
-#endif
+import           Test.QuickCheck (forAll,choose,Gen)
 
 import           Stack.Dot
 
+dummyVersion :: Version
+dummyVersion = fromMaybe (error "dotspec: parser error") (parseVersionFromString "0.0.0.0")
+
 spec :: Spec
 spec = do
   let graph =
          Map.mapKeys pkgName
-       . fmap (Set.map pkgName)
+       . fmap (\p -> (Set.map pkgName p, Just dummyVersion))
        . Map.fromList $ [("one",Set.fromList ["base","free"])
                         ,("two",Set.fromList ["base","free","mtl","transformers","one"])
                         ]
@@ -34,37 +36,42 @@ spec = do
       resolveDependencies (Just 0) graph stubLoader `shouldBe` return graph
 
     it "with depth 1, more dependencies are resolved" $ do
-      let graph' = Map.insert (pkgName "cycle") (Set.singleton (pkgName "cycle")) graph
+      let graph' = Map.insert (pkgName "cycle")
+                              (Set.singleton (pkgName "cycle"), Just dummyVersion)
+                              graph
           resultGraph = runIdentity (resolveDependencies (Just 0) graph stubLoader)
           resultGraph' = runIdentity (resolveDependencies (Just 1) graph' stubLoader)
       Map.size resultGraph < Map.size resultGraph' `shouldBe` True
 
     it "cycles are ignored" $ do
-       let graph' = Map.insert (pkgName "cycle") (Set.singleton (pkgName "cycle")) graph
+       let graph' = Map.insert (pkgName "cycle")
+                               (Set.singleton (pkgName "cycle"), Just dummyVersion)
+                                graph
            resultGraph = resolveDependencies Nothing graph stubLoader
            resultGraph' = resolveDependencies Nothing graph' stubLoader
        fmap Map.size resultGraph' `shouldBe` fmap ((+1) . Map.size) resultGraph
 
-#if MIN_VERSION_QuickCheck(2,8,0)
+    let graphElem e = Set.member e . Set.unions . Map.elems
+
     prop "requested packages are pruned" $ do
       let resolvedGraph = runIdentity (resolveDependencies Nothing graph stubLoader)
-          allPackages g = Set.map show (Map.keysSet g `Set.union` F.fold g)
+          allPackages g = Set.map show (Map.keysSet g `Set.union`  F.fold (fmap fst g))
       forAll (sublistOf (Set.toList (allPackages resolvedGraph))) $ \toPrune ->
         let pruned = pruneGraph [pkgName "one", pkgName "two"] toPrune resolvedGraph
         in Set.null (allPackages pruned `Set.intersection` Set.fromList toPrune)
 
     prop "pruning removes orhpans" $ do
       let resolvedGraph = runIdentity (resolveDependencies Nothing graph stubLoader)
-          allPackages g = Set.map show (Map.keysSet g `Set.union` F.fold g)
+          allPackages g = Set.map show (Map.keysSet g `Set.union` F.fold (fmap fst g))
           orphans g = Map.filterWithKey (\k _ -> not (graphElem k g)) g
       forAll (sublistOf (Set.toList (allPackages resolvedGraph))) $ \toPrune ->
         let pruned = pruneGraph [pkgName "one", pkgName "two"] toPrune resolvedGraph
-        in null (Map.keys (orphans pruned) \\ [pkgName "one", pkgName "two"])
-#endif
-
-  where graphElem e graph = Set.member e . Set.unions . Map.elems $ graph
+        in null (Map.keys (orphans (fmap fst pruned)) \\ [pkgName "one", pkgName "two"])
 
 {- Helper functions below -}
+-- Backport from QuickCheck 2.8 to 2.7.6
+sublistOf :: [a] -> Gen [a]
+sublistOf = filterM (\_ -> choose (False, True))
 
 -- Unsafe internal helper to create a package name
 pkgName :: ByteString -> PackageName
@@ -73,8 +80,8 @@ pkgName = fromMaybe failure . parsePackageName
    failure = error "Internal error during package name creation in DotSpec.pkgName"
 
 -- Stub, simulates the function to load package dependecies
-stubLoader :: PackageName -> Identity (Set PackageName)
-stubLoader name = return . Set.fromList . map pkgName $ case show name of
+stubLoader :: PackageName -> Identity (Set PackageName, Maybe Version)
+stubLoader name = return . (, Just dummyVersion) . Set.fromList . map pkgName $ case show name of
   "StateVar" -> ["stm","transformers"]
   "array" -> []
   "bifunctors" -> ["semigroupoids","semigroups","tagged"]
